import sys, argparse
from xtermcolor.ColorMap import XTermColorMap, VT100ColorMap

def cPrintfString(xterm):
  return '"\\033[38;5;{xterm:d}m%s\\033[0m"'.format(xterm=xterm)

def Cli():
  parser = argparse.ArgumentParser(
              description = 'xtermcolor: 256 terminal color library')

  parser.add_argument('action',
              choices = ['convert', 'list'],
              help = 'Actions')

  parser.add_argument('--color',
              help = 'Color to convert')

  parser.add_argument('--compat',
              choices=['xterm', 'vt100'],
              default='xterm',
              help = 'Compatibility mode.  Defaults to xterm.')

  cli = parser.parse_args()

  if cli.compat.lower() not in ['xterm', 'vt100']:
    sys.stderr.write('Error: --compat must be xterm or vt100\n')
    sys.exit(-1)
  
  if cli.compat.lower() == 'xterm':
    colorMap = XTermColorMap()
  if cli.compat.lower() == 'vt100':
    colorMap = VT100ColorMap()

  if cli.action == 'list':
    for xterm, hexcolor in colorMap.getColors().items():
      string = "\033[38;5;{xterm:d}mansi={xterm:d}; rgb=#{hexcolor:06x}; printf={cprintf:s}\033[0m"
      print(string.format(xterm=xterm, hexcolor=hexcolor, cprintf=cPrintfString(xterm)));

  if cli.action == 'convert':
    if not cli.color:
      sys.stderr.write('Error: must specify --color\n')
      sys.exit(-1)
    if cli.color[0] == '#':
      cli.color = '0x' + cli.color[1:]
    cli.color = int(cli.color, 16)
    
    colorizedMsg = colorMap.colorize('The quick brown fox jumped over the lazy dog.', cli.color)
    (ansi, rgb) = colorMap.convert(cli.color)
    print("#{start:06x} is closest to #{closest:06x} which is ANSI code {ansi:d}\n".format( \
          start=cli.color, closest=rgb, ansi=ansi))
    print("Example: " + colorizedMsg)
    print("printf() string: " + cPrintfString(ansi))



class TerminalColorMapException(Exception):
  pass

class TerminalColorMap:
  def getColors(self, order='rgb'):
    return self.colors

  def rgb(self, color):
    return ((color >> 16) & 0xff, (color >> 8) & 0xff, color & 0xff)

  def diff(self, color1, color2):
    (r1, g1, b1) = self.rgb(color1)
    (r2, g2, b2) = self.rgb(color2)
    return abs(r1-r2) + abs(g1-g2) + abs(b1-b2)

  def convert(self, hexcolor):
    diffs = {}
    for xterm, rgb in self.colors.items():
      diffs[self.diff(rgb, hexcolor)] = xterm
    minDiffAnsi = diffs[min(diffs.keys())]
    return (minDiffAnsi, self.colors[minDiffAnsi])

  def colorize(self, string, rgb=None, ansi=None):
    if rgb is None and ansi is None:
      raise TerminalColorMapException('colorize: must specify one named parameter: rgb or ansi')
    if rgb is not None and ansi is not None:
      raise TerminalColorMapException('colorize: must specify only one named parameter: rgb or ansi')

    if rgb:
      (closestAnsi, closestRgb) = self.convert(rgb)
    elif ansi:
      (closestAnsi, closestRgb) = (ansi, self.colors[ansi])

    return "\033[38;5;{ansiCode:d}m{string:s}\033[0m".format(ansiCode=closestAnsi, string=string)

class VT100ColorMap(TerminalColorMap):
  primary = [
    0x000000, 0x800000, 0x008000, 0x808000, 0x000080, 0x800080, 0x008080, 0xc0c0c0
  ]

  bright = [
    0x808080, 0xff0000, 0x00ff00, 0xffff00, 0x0000ff, 0xff00ff, 0x00ffff, 0xffffff
  ]

  def __init__(self):
    self.colors = dict()
    self._compute()

  def _compute(self):
    for index, color in enumerate(self.primary + self.bright):
      self.colors[index] = color

class XTermColorMap(VT100ColorMap):
  grayscale_start = 0x08;
  grayscale_end = 0xf8;
  grayscale_step = 10;
  intensities = [
    0x00, 0x5F, 0x87, 0xAF, 0xD7, 0xFF
  ]

  def _compute(self):
    for index, color in enumerate(self.primary + self.bright):
      self.colors[index] = color

    c = 16
    for i in self.intensities: 
      color = i << 16;
      for j in self.intensities: 
        color &= ~(0xff << 8);
        color |= j << 8;
        for k in self.intensities: 
          color &= ~0xff;
          color |= k;
          self.colors[c] = color
          c += 1

    c = 232    
    for hex in list(range(self.grayscale_start, self.grayscale_end, self.grayscale_step)):
      color = (hex << 16) | (hex << 8) | hex;
      self.colors[c] = color
      c += 1



import os

from xtermcolor import distribute_setup
distribute_setup.use_setuptools()

from setuptools import setup

version = '1.0.4'
README = os.path.join(os.path.dirname(__file__), 'README')
long_description = open(README).read()

setup(
  name='xtermcolor',
  version=version,
  description=long_description,
  author='Scott Frazer',
  author_email='sfrazer@broadinstitute.org',
  packages=['xtermcolor'],
  package_data={'xtermcolor': ['distribute_setup.py']},
  install_requires=[],
  entry_points={
    'console_scripts': [
      'xtermcolor = xtermcolor.Main:Cli'
    ]
  },
  license = "MIT",
  keywords = "xterm, ANSI, xterm-256, terminal, color",
  url = "http://github.com/broadinstitute/xtermcolor",
  classifiers=[
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Natural Language :: English',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.0',
    'Programming Language :: Python :: 3.1',
    'Programming Language :: Python :: 3.2',
    'Programming Language :: Python :: 3.3',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Terminals :: Terminal Emulators/X Terminals',
    'Topic :: Utilities'
  ]
)
